/*
    UHU-menu -- output plugin for XDG-compilant systems, e.g. KDE (3.2), Gnome (2.12)
    Copyright (C) 2003, 2005  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void dump (const struct item *item, FILE *f, int n)
{
	int i;
	char *p;

	if (!item->issubmenu) return;
	p = strrchr(item->x_uhu_menu, '/');
	if (p == NULL) p = item->x_uhu_menu; else p++;
	if (p[0] == '\0') p = "UHU";
	indent(f, n);
	fprintf(f, "<Menu>\n");
	indent(f, n+1);
	fprintf(f, "<Name>");
	p = get_value(item, NULL, "Name", "");
	while (*p) {
		if ((*p >= '0' && *p <= '9') ||
		    (*p >= 'A' && *p <= 'Z') ||
		    (*p >= 'a' && *p <= 'z')) fputc(*p, f);
		else fputc('_', f);
		p++;
	}
	fprintf(f, "</Name>\n");
	if (n == 0) {
		indent(f, n+1);
		fprintf(f, "<DefaultAppDirs/>\n");
		indent(f, n+1);
		fprintf(f, "<DefaultDirectoryDirs/>\n");
	}
	indent(f, n+1);
	fprintf(f, "<Directory>%s</Directory>\n", item->file_name);
	for (i = 0; i < item->subitems_nr; i++) {
		dump(&item->subitems[i], f, n+1);
	}
	indent(f, n+1);
	fprintf(f, "<Include>\n");
	indent(f, n+2);
	fprintf(f, "<Category>X-UHU-Menu%s</Category>\n", item->x_uhu_menu);
	indent(f, n+1);
	fprintf(f, "</Include>\n");
	indent(f, n);
	fprintf(f, "</Menu>\n");
}

int write_menu (const struct item *item, const char *path,
  struct keep_files *keep_files)
{
	char file[PATH_MAX];
	char file_tmp[PATH_MAX];
	FILE *f;

	sprintf(file, "%s/UHU.menu", path);
	sprintf(file_tmp, "%s..tmp", file);
	f = fopen(file_tmp, "w");
	if ((f = fopen(file_tmp, "w")) == NULL) {
		message_e(3, "fopen %s", file_tmp);
		return -1;
	}
	fprintf(f, "<!DOCTYPE Menu PUBLIC \"-//freedesktop//DTD Menu 1.0//EN\" \"http://www.freedesktop.org/standards/menu-spec/1.0/menu.dtd\">\n");
	if (item->subitems_nr > 0) dump(item, f, 0);
	fclose(f);
	move(file_tmp, file);
	keep_file(keep_files, "UHU.menu");
	return 0;
}

int need_all_entries = 1;

